package com.jjt.biz.service.impl;

import com.jjt.biz.service.IApiYrService;
import com.jjt.biz.vo.YrEnergyTradeVO;
import com.jjt.biz.vo.YrProdEfficiencyVO;
import com.jjt.biz.vo.YrProdLineStatusVO;
import com.jjt.biz.vo.YrProdTradeVO;
import com.jjt.common.core.redis.RedisCache;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

/**
 * 首页统计数据
 *
 * @author wukai
 * @date 2024/5/4 20:35
 */
@Service
@Slf4j
public class ApiYrServiceImpl implements IApiYrService {
    @Resource
    private RedisCache redisCache;

    /**
     * 获取产线状态
     *
     * @return 结果
     */
    @Override
    public YrProdLineStatusVO status() {
        YrProdLineStatusVO vo = new YrProdLineStatusVO();
        vo.mock();
        return vo;
    }

    /**
     * 获取生产效率
     *
     * @return 结果
     */
    @Override
    public List<YrProdEfficiencyVO> eff() {
        List<YrProdEfficiencyVO> list = new ArrayList<>();
        YrProdEfficiencyVO[] arr = new YrProdEfficiencyVO[9];
        for (int i = 0; i < 9; i++) {
            YrProdEfficiencyVO vo = new YrProdEfficiencyVO(i);
            vo.mock();
            arr[i] = vo;
        }
        YrProdEfficiencyVO line0 = new YrProdEfficiencyVO(0);
        for (int i = 1; i < 9; i++) {
            YrProdEfficiencyVO vo = arr[i];
            line0.setEffA(line0.getEffA().add(vo.getEffA()));
            line0.setEffB(line0.getEffB().add(vo.getEffB()));
            line0.setTimeA(line0.getTimeA().add(vo.getTimeA()));
            line0.setTimeB(line0.getTimeB().add(vo.getTimeB()));
            line0.setLengthA(line0.getLengthA().add(vo.getLengthA()));
            line0.setLengthB(line0.getLengthB().add(vo.getLengthB()));
            line0.setWeightA(line0.getWeightA().add(vo.getWeightA()));
            line0.setWeightB(line0.getWeightB().add(vo.getWeightB()));
            line0.setWeight(line0.getWeight().add(vo.getWeight()));
            line0.setLength(line0.getLength().add(vo.getLength()));

        }

        line0.setEffA(line0.getEffA().divide(BigDecimal.valueOf(8), 2, RoundingMode.HALF_UP));
        line0.setEffB(line0.getEffB().divide(BigDecimal.valueOf(8), 2, RoundingMode.HALF_UP));

        list.add(line0);
        for (int i = 1; i < 9; i++) {
            YrProdEfficiencyVO vo = arr[i];
            list.add(vo);
        }
        return list;
    }

    /**
     * 获取生产趋势
     *
     * @return 结果
     */
    @Override
    public List<YrProdTradeVO> prodTrade() {
        return mockProd();
    }

    /**
     * 获取能耗趋势
     *
     * @return 结果
     */
    @Override
    public List<YrEnergyTradeVO> energyTrade() {
        return mockEnergy();
    }

    private List<YrProdTradeVO> mockProd() {
        List<YrProdTradeVO> list = new ArrayList<>();
        LocalDate end = LocalDate.now().minusDays(1);
        LocalDate start = end.minusDays(30);
        do {
            YrProdTradeVO vo = new YrProdTradeVO();
            vo.setDate(start.getDayOfMonth() + "");
            vo.setTips(start.toString());
            vo.mock();
            list.add(vo);
            start = start.plusDays(1);
        } while (!start.isAfter(end));
        return list;

    }

    private List<YrEnergyTradeVO> mockEnergy() {
        List<YrEnergyTradeVO> list = new ArrayList<>();
        LocalDate end = LocalDate.now().minusDays(1);
        LocalDate start = end.minusDays(30);
        do {
            YrEnergyTradeVO vo = new YrEnergyTradeVO();
            vo.setDate(start.getDayOfMonth() + "");
            vo.setTips(start.toString());
            vo.mock();
            list.add(vo);
            start = start.plusDays(1);
        } while (!start.isAfter(end));
        return list;

    }

    /**
     * 获取周几
     *
     * @param date
     * @return
     */
    private String week(LocalDate date) {
        String[] weeks = {"一", "二", "三", "四", "五", "六", "日"};
        return "周" + weeks[date.getDayOfWeek().getValue() - 1];
    }

    public static void main(String[] args) {
        int day = 7;
        LocalDate end = LocalDate.now().minusDays(1);
        LocalDate start = end.minusDays(day - 1);
        do {
            System.err.println(start + "\t" + start.getDayOfWeek().getValue());
            start = start.plusDays(1);
        } while (!start.isAfter(end));
    }
}
