package com.jjt.calc.service.impl;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.jjt.biz.domain.TwinDeviceYhj;
import com.jjt.biz.service.ITwinDeviceYhjService;
import com.jjt.calc.domain.TwinCalcHourYhj;
import com.jjt.calc.mapper.TwinCalcHourYhjMapper;
import com.jjt.calc.service.ITwinCalcDayYhjService;
import com.jjt.calc.service.ITwinCalcHourYhjService;
import com.jjt.common.utils.DateUtils;
import com.jjt.utils.IotService;
import com.jjt.utils.Tools;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneOffset;
import java.util.*;

/**
 * 印花机1小时统计数据Service业务层处理
 *
 * @author wukai
 * @date 2025-03-06
 */
@Service
@Slf4j
public class TwinCalcHourYhjServiceImpl implements ITwinCalcHourYhjService {
    @Resource
    private TwinCalcHourYhjMapper twinCalcHourYhjMapper;
    @Resource
    private IotService iotService;
    @Resource
    private SqlSessionFactory factory;
    @Resource
    private ITwinDeviceYhjService yhjService;
    @Resource
    private ITwinCalcDayYhjService dayYhjService;

    /**
     * 查询印花机1小时统计数据
     *
     * @param id 印花机1小时统计数据主键
     * @return 印花机1小时统计数据
     */
    @Override
    public TwinCalcHourYhj selectTwinCalcHourYhjById(Long id) {
        return twinCalcHourYhjMapper.selectTwinCalcHourYhjById(id);
    }

    /**
     * 查询印花机1小时统计数据列表
     *
     * @param twinCalcHourYhj 印花机1小时统计数据
     * @return 印花机1小时统计数据
     */
    @Override
    public List<TwinCalcHourYhj> selectTwinCalcHourYhjList(TwinCalcHourYhj twinCalcHourYhj) {
        return twinCalcHourYhjMapper.selectTwinCalcHourYhjList(twinCalcHourYhj);
    }

    /**
     * 新增印花机1小时统计数据
     *
     * @param twinCalcHourYhj 印花机1小时统计数据
     * @return 结果
     */
    @Override
    public int insertTwinCalcHourYhj(TwinCalcHourYhj twinCalcHourYhj) {
        return twinCalcHourYhjMapper.insertTwinCalcHourYhj(twinCalcHourYhj);
    }

    /**
     * 修改印花机1小时统计数据
     *
     * @param twinCalcHourYhj 印花机1小时统计数据
     * @return 结果
     */
    @Override
    public int updateTwinCalcHourYhj(TwinCalcHourYhj twinCalcHourYhj) {
        return twinCalcHourYhjMapper.updateTwinCalcHourYhj(twinCalcHourYhj);
    }

    /**
     * 批量删除印花机1小时统计数据
     *
     * @param ids 需要删除的印花机1小时统计数据主键
     * @return 结果
     */
    @Override
    public int deleteTwinCalcHourYhjByIds(Long[] ids) {
        return twinCalcHourYhjMapper.deleteTwinCalcHourYhjByIds(ids);
    }

    /**
     * 删除印花机1小时统计数据信息
     *
     * @param id 印花机1小时统计数据主键
     * @return 结果
     */
    @Override
    public int deleteTwinCalcHourYhjById(Long id) {
        return twinCalcHourYhjMapper.deleteTwinCalcHourYhjById(id);
    }

    /**
     * 统计上一时段数据
     */
    @Override
    public void last() {
        LocalDateTime ldt = Tools.currWholeTime();
        //上一个小时
        ldt = ldt.minusHours(1);
        //开始时间需要向前取一秒
        LocalDateTime start = ldt;
        LocalDateTime end = ldt.plusHours(1);
        calc(start, end);
    }

    /**
     * 按指定时间统计
     *
     * @param start 开始时间
     * @param end   结束时间
     */
    @Override
    public void calc(LocalDateTime start, LocalDateTime end) {
        Long startTime = start.toInstant(ZoneOffset.of("+8")).toEpochMilli();
        Long endTime = end.toInstant(ZoneOffset.of("+8")).toEpochMilli();
        deleteTwinCalcHourYhj(DateUtils.toDate(start.toLocalDate()), start.getHour());
        List<TwinDeviceYhj> list = yhjService.selectTwinDeviceYhjList(new TwinDeviceYhj());
        List<TwinCalcHourYhj> calcList = new ArrayList<>();
        //  使用设备编码作为Map的key, 使用设备对象本身作为Map的value, 如果有重复的key，保留现有的value
        Map<String, TwinDeviceYhj> yhjMap = new HashMap<>(16);
        List<String> fieldList = new ArrayList<>();
        String prefix = "root.tl.suxi";
        String[] fields = {"Formula_data_set_1", "Formula_data_act_5"};
        for (TwinDeviceYhj yhj : list) {
            for (String field : fields) {
                String tmp = yhj.getDeviceCode() + "." + field;
                fieldList.add(tmp);
                yhjMap.put(prefix + "." + tmp, yhj);
            }
        }
        String sql = "select %s from " + prefix + " where time>%s and time <=%s";

        sql = String.format(sql, String.join(",", fieldList), startTime, endTime);
        JSONObject jsonObject = iotService.query(sql);
        JSONObject data = jsonObject.getJSONObject("data");
        JSONArray columnNames = data.getJSONArray("columnNames");
        JSONArray values = data.getJSONArray("values");
        JSONArray timestamps = data.getJSONArray("timestamps");
        //记录上一次记录的值
        Integer[] last = new Integer[columnNames.size()];
        //记录最后一次的值，防中间有数，后面没数
        Integer[] over = new Integer[columnNames.size()];
        //当前记录值
        Integer[] total = new Integer[columnNames.size()];
        //记录第一次记录的值，如果清0，则记录清零时刻的值
        Integer[] first = new Integer[columnNames.size()];
        //记录统计值
        Integer[] curr = new Integer[columnNames.size()];
        for (int i = 0; i < values.size(); i++) {
            JSONArray da = values.getJSONArray(i);
            da.toList(Integer.class).toArray(curr);
            if (i == 0) {
                cloneArray(curr, last);
                cloneArray(curr, first);
                cloneArray(curr, over);
                continue;
            }
            for (int j = 0; j < columnNames.size(); j++) {
                String name = columnNames.getStr(j);
                if (name.contains("Formula_data_set_1")) {
                    //如果是版距，则需要记录变化
                    if (curr[j] != null && last[j] != null && curr[j].intValue() != last[j].intValue()) {
                        //这里要用j+1才行
                        calcLength(j + 1, last, first, total);
                        combo(calcList, start, yhjMap.get(name), last[j], total[j + 1]);
                        total[j + 1] = 0;
                    }
                }
                if (name.contains("Formula_data_act_5")) {
                    //如果是米长，则需要记录是否清0
                    //如果当前值为小于上一个，且上一个值不为0，则计算
                    //因为会出现数据波动，停机再启动之后的第一个点不为0，为0.00几几几的
                    if (curr[j] != null && last[j] != null && curr[j] < last[j] && last[j] != 0) {
                        calcLength(j, last, first, total);
                        cloneArray(curr, first);
                    }
                }
            }
            //将本次的值，设为上一条记录了
            cloneArray(curr, last);
            cloneArray(curr, over);
            cloneArrayFirst(curr, first);
        }
        for (int j = 0; j < columnNames.size(); j++) {
            String name = columnNames.getStr(j);
            if (name.contains("Formula_data_set_1")) {
                calcLength(j + 1, over, first, total);
                combo(calcList, start, yhjMap.get(name), over[j], total[j + 1]);
            }
        }

        try (SqlSession sqlSession = factory.openSession(ExecutorType.BATCH, false)) {
            if (calcList.size() > 0) {
                TwinCalcHourYhjMapper mapper = sqlSession.getMapper(TwinCalcHourYhjMapper.class);
                calcList.forEach(mapper::insertTwinCalcHourYhj);
                sqlSession.commit();
            }
        }
    }

    /**
     * 查询数据库最后一条记录
     *
     * @return 结果
     */
    @Override
    public TwinCalcHourYhj lastRecord() {
        return twinCalcHourYhjMapper.lastRecord();
    }

    /**
     * 补录数据至当前时间
     */
    @Override
    public void calc2Curr() {
        TwinCalcHourYhj last = lastRecord();
        LocalDate localDate = DateUtils.toLocalDate(last.getDataDate());
        LocalDateTime start = LocalDateTime.of(localDate, LocalTime.MIN).plusHours(last.getHour() + 1);
        LocalDateTime end = start.plusHours(1);

        LocalDateTime stop = Tools.currWholeTime();
        while (!end.isAfter(stop)) {
            calc(start, end);
            log.info("补录数据===========start:{},end:{},stop:{}", start, end, stop);
            if (end.getHour() == 7) {
                //跨天,统计前一天的总数据
                log.info("----------------{},{}", start, end);
                dayYhjService.day(start.toLocalDate().minusDays(1));
            }
            start = end;
            end = end.plusHours(1);
        }
    }

    /**
     * 按时间和小时删除数据
     *
     * @param date 时间
     * @param hour 小时
     */
    public void deleteTwinCalcHourYhj(Date date, int hour) {
        twinCalcHourYhjMapper.deleteTwinCalcHourYhj(date, hour);
    }

    /**
     * 初始值，如果当前值不为空，并且原来值为空
     */
    void cloneArrayFirst(Integer[] arr, Integer[] target) {
        for (int i = 0; i < arr.length; i++) {
            if (arr[i] != null && target[i] == null) {
                target[i] = arr[i];
            }
        }
    }

    /**
     * 复制数组，如果当前值不为空
     */
    void cloneArray(Integer[] arr, Integer[] target) {
        for (int i = 0; i < arr.length; i++) {
            if (arr[i] != null) {
                target[i] = arr[i];
            }
        }
    }

    /**
     * 组合入库数据
     */
    void combo(List<TwinCalcHourYhj> calcList, LocalDateTime start, TwinDeviceYhj yhj, Integer distance, Integer length) {
        if (distance != null && length != null) {
            TwinCalcHourYhj calc = new TwinCalcHourYhj();
            calc.setDeviceId(yhj.getDeviceId());
            calc.setDataDate(DateUtils.toDate(start.toLocalDate()));
            calc.setHour(start.getHour());
            calc.setDistance(distance);
            calc.setLength(length);
            calcList.add(calc);
        }
    }

    /**
     * 计算米长
     */
    void calcLength(int j, Integer[] last, Integer[] first, Integer[] total) {
        if (last[j] != null && first[j] != null) {
            if (total[j] == null) {
                total[j] = 0;
            }
            total[j] += (last[j] - first[j]);
        }
    }
}
