package com.jjt.order.service;

import com.jjt.order.domain.TwinOrder;

import java.time.LocalDate;
import java.util.List;

/**
 * 销售订单信息Service接口
 *
 * @author wukai
 * @date 2025-01-16
 */
public interface ITwinOrderService {
    /**
     * 查询销售订单信息
     *
     * @param oderId 销售订单信息主键
     * @return 销售订单信息
     */
    public TwinOrder selectTwinOrderByOderId(Long oderId);

    /**
     * 查询销售订单信息列表
     *
     * @param twinOrder 销售订单信息
     * @return 销售订单信息集合
     */
    public List<TwinOrder> selectTwinOrderList(TwinOrder twinOrder);

    /**
     * 新增销售订单信息
     *
     * @param twinOrder 销售订单信息
     * @return 结果
     */
    public int insertTwinOrder(TwinOrder twinOrder);

    /**
     * 修改销售订单信息
     *
     * @param twinOrder 销售订单信息
     * @return 结果
     */
    public int updateTwinOrder(TwinOrder twinOrder);

    /**
     * 批量删除销售订单信息
     *
     * @param oderIds 需要删除的销售订单信息主键集合
     * @return 结果
     */
    public int deleteTwinOrderByOderIds(String oderIds);

    /**
     * 删除销售订单信息信息
     *
     * @param oderId 销售订单信息主键
     * @return 结果
     */
    public int deleteTwinOrderByOderId(Long oderId);

    /**
     * 按时间获取订单入库
     *
     * @param localDate 时间
     */
    void getOrder(LocalDate localDate);

    /**
     * 按时间获取订单信息
     *
     * @param start
     * @param end
     * @return
     */
    List<TwinOrder> selectTwinOrderListByDate(String start, String end);
}
